SystemOrganization addCategory: #VisuAule!
SystemOrganization addCategory: #'VisuAule-Objects'!
SystemOrganization addCategory: #'VisuAule-Objects-Tests'!
SystemOrganization addCategory: #'VisuAule-Example'!

TestCase subclass: #VABookAndPersonTest
	instanceVariableNames: 'book ralph eric joe'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'VisuAule-Objects-Tests'!

!VABookAndPersonTest methodsFor: 'running' stamp: 'CamiloGomez 7/6/2012 18:39'!
setUp
	
	book := VABook new name: 'design patterns'.
	ralph := VAPerson new name: 'Ralph Johnson'.
	eric := VAPerson new name: 'Eric Gamma'.
	book addAuthor: ralph; addAuthor: eric.
	
	joe := VAReader new.
	joe readBook: book.! !

!VABookAndPersonTest methodsFor: 'testing scenarios' stamp: 'CamiloGomez 7/5/2012 12:12'!
testPerson

	"The scenario we are here simulating is (person visualize)"
	
	| view |
	
	view := ROView new.
	
	ralph visualizeOn: view.
	eric visualizeOn: view.
	
	"view open"
	
	self assert: view elements size = 2! !

!VABookAndPersonTest methodsFor: 'testing scenarios' stamp: 'CamiloGomez 7/5/2012 12:12'!
testVisualizingBooksAndPersons

	"The scenario we are here simulating is (book new visualize)"
	
	| view |
		
	view := ROView new.
	
	self assert: view elements isEmpty.
	book visualizeOn: view.
	"view open"
	self assert: view elements size = 5.
	self assert: (view elementFromModel: book) notNil.
	self assert: (view elementFromModel: ralph) notNil.
	self assert: (view elementFromModel: eric) notNil.
	self assert: (view elementFromModel: joe) isNil.! !

TestCase subclass: #VAMovieTest
	instanceVariableNames: 'lordOfTheRings'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'VisuAule-Objects-Tests'!

!VAMovieTest methodsFor: 'running' stamp: 'CamiloGomez 7/6/2012 16:52'!
setUp

	lordOfTheRings := VAMovie new.
	
	lordOfTheRings title: 'The Lord Of The Rings: The Fellowship Of The Ring'. 
	lordOfTheRings director: (VAPerson new name: 'Peter Jackson').
	lordOfTheRings duration: '3 hours'.
	lordOfTheRings country: 'New Zealand'.
	lordOfTheRings genre: 'Fantasy'.
	! !

!VAMovieTest methodsFor: 'tests' stamp: 'CamiloGomez 7/6/2012 16:58'!
testVisualizeMovie

	| view |
	
	view := ROView new.
	
	self assert: view elements isEmpty.
	
	lordOfTheRings visualizeOn: view.
	
	self assert: view elements size = 3.
	self assert: (view elementFromModel: lordOfTheRings) notNil.
	! !

TestCase subclass: #VAVisualizingPointTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'VisuAule-Objects-Tests'!

!VAVisualizingPointTest methodsFor: 'testing scenarios' stamp: 'CamiloGomez 7/6/2012 16:37'!
testVisualizingPoint

	"The scenario we are here simulating is (2@4) visualize"
	
	| view |
	view := ROView new.
	
	self assert: view elements isEmpty.
	(2 @ 4) visualizeOn: view.
	(10 @ 10) visualizeOn: view.
	"view open."
	self assert: view elements size = 2.! !

!Browser methodsFor: '*visuaule-objects' stamp: 'CamiloGomez 7/6/2012 12:23'!
interactionFor: element on: view

	super interactionFor: element on: view.
	
	element on: ROMouseLeftClick do: [ :event | Browser open].! !

!Number methodsFor: '*visuaule-objects' stamp: 'CamiloGomez 7/25/2012 16:27'!
elementsNotAdded 

	^ ROElement new.! !

!Number methodsFor: '*visuaule-objects' stamp: 'CamiloGomez 7/25/2012 16:18'!
roassalElementFor: object
	| element |
	
	element := ROElement on: object.
	element + ROLabel + ROBorder.
	element translateTo: 5@5.
	element @ ROLightlyHighlightable @ RODraggable.
	
	^ element! !

!Number methodsFor: '*visuaule-objects' stamp: 'CamiloGomez 6/27/2012 15:38'!
shouldBeAddedAsElement
	^ false! !

!UndefinedObject methodsFor: '*visuaule-objects' stamp: 'CamiloGomez 6/27/2012 15:50'!
shouldBeAddedAsElement
	^ false! !

!UndefinedObject methodsFor: '*visuaule-objects' stamp: 'CamiloGomez 7/17/2012 17:15'!
shouldBeAddedAsElementOnEnter
	^ false! !

!ROEaselMorphic methodsFor: '*visuaule-objects' stamp: 'CamiloGomez 7/6/2012 17:12'!
interactionFor: element on: view

	super interactionFor: element on: view.
	
	element on: ROMouseLeftClick do: [ :event | ROEaselMorphic new open]. ! !

!Boolean methodsFor: '*visuaule-objects' stamp: 'CamiloGomez 7/17/2012 17:23'!
shouldBeAddedAsElement 
	^ false! !

!ThreadSafeTranscript methodsFor: '*visuaule-objects' stamp: 'CamiloGomez 7/6/2012 15:55'!
interactionFor: element on: view

	super interactionFor: element on: view.
	
	element on: ROMouseLeftClick do: [ :event | Transcript open].! !

!Object methodsFor: '*visuaule-objects' stamp: 'CamiloGomez 8/22/2012 15:37'!
addElement: el of: elementForSelf on: view

	" Add the element 'el', set the interaction for 'el', and add an Edge from 'elementForSelf' to 'el' on 'view' "

	"Add the interaction" 
	(el model) interactionFor: el on: view.
	
	"Add the element and a ROEdge"
	 view add: el.
	" view shapesDo: (ROOrthoVerticalLineShape new)."
	 view add: (ROEdge lineFrom: elementForSelf to: el)! !

!Object methodsFor: '*visuaule-objects' stamp: 'CamiloGomez 8/14/2012 15:49'!
addReferencedObjectsOf: elementForSelf on: view

	" Add the variables of elementForSelf on view, unless it's a Collection, in which case, do handleCollection "

	"Go over all the Instance Variables of self" 
	self class allInstVarNames do: [ :varName |
		| el object |
		"Identify the object of varName"
		object := self instVarNamed: varName.
		object shouldBeAddedAsElement
			ifTrue: 
				[object isCollection
					"If it's a Collection, handle it as a Collection"
					ifTrue: [self handleCollectionIn: object of: elementForSelf on: view]
					"If not, add the ROElement of object on view"
					ifFalse: ["el := (elementForSelf model) roassalElementFor: object."
							el := (elementForSelf model) roassalElementFor: object withName: varName. 
				 			(elementForSelf model) addElement: el of: elementForSelf on: view.]]] ! !

!Object methodsFor: '*visuaule-objects' stamp: 'CamiloGomez 8/22/2012 13:23'!
elementsNotAdded

	" Return all the elements that will be poped up when the mouse is on top of elementForSelf "

	| otherElements elementObject newElement varNames varNamesElement otherElementsElement bothColumns varNameElement elementTemp |
	
	bothColumns := OrderedCollection new.
	varNamesElement := ROElement new.
	otherElementsElement := ROElement new.
	newElement := ROElement new.
	varNames := OrderedCollection new.
	otherElements := OrderedCollection new.
	varNameElement := ROElement new.
	
	self class allInstVarNames do: [ :varName |
		| el object |
		object := self instVarNamed: varName.
		elementObject := self roassalOtherElementFor: object.
		object shouldBeAddedAsElementOnEnter
		ifTrue: [ 
			"object notNil 
			ifTrue: ["
				varNameElement := ((ROElement on: varName) + ROLabel).
				object shouldBeAddedAsElement
				ifTrue: [
				varNameElement shapesDo: [ :d | d color: Color red].
				"elementObject shapesDo: [ :d | d color: Color red]"
				].
				varNames add: varNameElement.
				object isCollection & object shouldBeAddedAsElement
				ifFalse: [otherElements add: elementObject]
				
				ifTrue: [
					"elementTemp := (self roassalOtherElementFor: object class)."
					elementTemp := self roassalOtherElementFor: object withName: ('a ',(object class) asString).
					elementTemp shapesDo: [ :d | d color: Color green].
					otherElements add: elementTemp]
				"]"
			]].

	ROVerticalLineLayout on: varNames.
	ROVerticalLineLayout on: otherElements.
	
	varNamesElement addAll: varNames.
	otherElementsElement addAll: otherElements.
	
	bothColumns add: varNamesElement.
	bothColumns add: otherElementsElement.
	ROHorizontalLineLayout on: bothColumns. 

	varNames isEmpty
	ifTrue: [ ^ ROElement new.]
	ifFalse: [
	newElement addAll: bothColumns.
	
	newElement + ROBorder.
	newElement shapesDo: [ :d | d color: Color blue].
	
	^ newElement]! !

!Object methodsFor: '*visuaule-objects' stamp: 'CamiloGomez 8/14/2012 15:52'!
handleCollectionIn: object of: elementForSelf on: view

	" Verifies if the Collection has enough numbers of elements to display them, if not, displays only the collection itself "

	| el |
	
	(object size < 5)
	ifTrue: [object do: [ :element | 
					element notNil
					ifTrue: [el := (elementForSelf model) roassalElementFor: element.
				 	(elementForSelf model) addElement: el of: elementForSelf on: view.]]]
	ifFalse: [el := (elementForSelf model) roassalElementFor: object.
			(elementForSelf model) addElement: el of: elementForSelf on: view.]
 ! !

!Object methodsFor: '*visuaule-objects' stamp: 'CamiloGomez 8/14/2012 15:36'!
interactionFor: elementForSelf on: view

	" Set the interaction of 'elementForSelf' "

	| otherElements elementObject |
	elementForSelf @ (ROMenuActivable new item: 'inspect element' action: #inspect;
									item: 'browse element class' action: #browse;
									item: 'visualize object' action: 
									[
										(elementForSelf model) visualize.	
									]
								).

	otherElements := (elementForSelf model) elementsNotAdded.
	
	elementForSelf on: ROMouseEnter do: [ :event | 
			view add: otherElements.
			otherElements translateTo: ((elementForSelf position) + (0@20))].
		
	elementForSelf on: ROMouseLeave do: [ :event | 
			view remove: otherElements]! !

!Object methodsFor: '*visuaule-objects' stamp: 'CamiloGomez 8/14/2012 15:36'!
roassalElementFor: object
	
	" Makes a ROElement with border and label on object "
	
	| element |
	
	element := ROElement on: object.
	element + (ROLabel new text: object class) + ROBorder.
	element translateTo: 5@5 .
	element @ ROLightlyHighlightable @ RODraggable.
	"element shapesDo: [ :d | d color: Color red]."
	
	^ element! !

!Object methodsFor: '*visuaule-objects' stamp: 'CamiloGomez 8/14/2012 15:37'!
roassalElementFor: object withName: aString

	" Makes a ROElement with border and name aString "

	| element |
	
	element := ROElement on: object.
	element + (ROLabel new text: aString) + ROBorder.
	element translateTo: 5@5 .
	element @ ROLightlyHighlightable @ RODraggable.
	"element shapesDo: [ :d | d color: Color red]."
	
	^ element! !

!Object methodsFor: '*visuaule-objects' stamp: 'CamiloGomez 8/14/2012 15:37'!
roassalOtherElementFor: object

	" Makes a ROElement without border, with label on object "

	| element |
	
	element := ROElement on: object.
	element + ROLabel.
	element @ ROLightlyHighlightable @ RODraggable.
	"element shapesDo: [ :d | d color: Color red]."
	
	^ element! !

!Object methodsFor: '*visuaule-objects' stamp: 'CamiloGomez 8/14/2012 15:38'!
roassalOtherElementFor: object withName: aString

	" Makes a ROElement without border, with label aString "

	| element |
	
	element := ROElement on: object.
	element + (ROLabel new text: aString).
	element @ ROLightlyHighlightable @ RODraggable.
	"element shapesDo: [ :d | d color: Color red]."
	
	^ element! !

!Object methodsFor: '*visuaule-objects' stamp: 'CamiloGomez 8/14/2012 15:38'!
setLayoutOn: view

	" Sets a GridLayout on view elements"

	ROGridLayout new 
				translator: (ROSmoothLayoutTranslator new);
				applyOn: view elements.! !

!Object methodsFor: '*visuaule-objects' stamp: 'CamiloGomez 8/14/2012 15:38'!
setThisLayout: layout on: elements

	" Set anyLayout of elements "

	layout new 
				translator: (ROSmoothLayoutTranslator new);
				applyOn: elements.
			
	"I don't know yet if this is gonna be necessary"! !

!Object methodsFor: '*visuaule-objects' stamp: 'CamiloGomez 7/17/2012 17:23'!
shouldBeAddedAsElement
	^ true! !

!Object methodsFor: '*visuaule-objects' stamp: 'CamiloGomez 7/17/2012 17:12'!
shouldBeAddedAsElementOnEnter
	^ true! !

!Object methodsFor: '*visuaule-objects' stamp: 'CamiloGomez 8/22/2012 18:05'!
visualize

	" visualize self on a ROView "

	| rawView view obj |
	
	rawView := ROView new.
	view := ROMondrianViewBuilder view: rawView.
	view shape: (ROOrthoVerticalLineShape new).
	
	"view @ RODraggable @ RODraggableWithVelocity."
	rawView on: ROMouseLeftClick do: [ :event | ROZoomInMove new on: view].
	rawView on: ROMouseRightClick do: [ :event | ROZoomOutMove new on: view].
	"view add: ((ROButtonElement named: 'Zoom in' do: [ROZoomInMove new on: view]) + ROBorder).
	view add: ((ROButtonElement named: 'Zoom out' do: [ROZoomOutMove new on: view]) + ROBorder)."	
	self visualizeOn: rawView.
	
	view open.
! !

!Object methodsFor: '*visuaule-objects' stamp: 'CamiloGomez 8/14/2012 16:24'!
visualizeOn: view  

	" Visualize self on view "

	"We add the object"
	 | elementForSelf |
	elementForSelf := self roassalElementFor: self.
	self interactionFor: elementForSelf on: view.
	view add: elementForSelf.
	
	"We set the title to the window"
	view title: 'VisuAule'.
	
	"We add the referenced objects"
	self addReferencedObjectsOf: elementForSelf on: view.
	
	"We set a Layout, in this case, gridLayout"
	self setThisLayout: ROTreeLayout on: (view elements). ! !

Object subclass: #VABook
	instanceVariableNames: 'authors title name'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'VisuAule-Objects'!

!VABook methodsFor: 'actions' stamp: 'CamiloGomez 6/27/2012 15:18'!
addAuthor: aPerson
	authors add: aPerson! !

!VABook methodsFor: 'initialize-release' stamp: 'CamiloGomez 6/27/2012 15:17'!
initialize
	super initialize.
	authors := OrderedCollection new.! !

!VABook methodsFor: 'accessing' stamp: 'CamiloGomez 6/27/2012 15:26'!
name: aString
	name := aString! !

Object subclass: #VAExample
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'VisuAule-Example'!

!VAExample commentStamp: '<historical>' prior: 0!
List of examples!

!VAExample methodsFor: 'open tools' stamp: 'CamiloGomez 7/6/2012 17:11'!
allTools

	"self new allTools"

	| view |
	
	view := ROView new.
	
	Browser new visualizeOn: view.
	Transcript visualizeOn: view.
	ROEaselMorphic new visualizeOn: view.
	
	view open.! !

!VAExample methodsFor: 'open tools' stamp: 'CamiloGomez 7/6/2012 12:20'!
browser
	"self new browser"

	Browser new visualize.! !

!VAExample methodsFor: 'open tools' stamp: 'CamiloGomez 7/6/2012 17:37'!
roassalEasel

	"self new roassalEasel"
	
	ROEaselMorphic new visualize.
	! !

!VAExample methodsFor: 'layout' stamp: 'CamiloGomez 7/6/2012 12:20'!
simpleLayout

	"self new simpleLayout"

	"Not implemented yet"! !

!VAExample methodsFor: 'open tools' stamp: 'CamiloGomez 7/6/2012 15:55'!
transcript

	"self new transcript"
	
	Transcript visualize.! !

!VAExample methodsFor: 'va objects' stamp: 'CamiloGomez 7/17/2012 17:25'!
visualizeVADirector 

	"self new visualizeVADirector"

	| director view |
	
	view := ROView new.
	
	director := VADirector new name: 'Francis Ford Coppola'.
	
	director visualizeOn: view.	
	
	view open.! !

!VAExample methodsFor: 'va objects' stamp: 'CamiloGomez 8/9/2012 16:24'!
visualizeVAMovie

	"self new visualizeVAMovie"

	| movie movie2 movie3 movie4 movie5 view director |
	
	view := ROView new.
	movie := VAMovie new.
	movie2 := VAMovie new.
	movie3 := VAMovie new.
	movie4 := VAMovie new.
	movie5 := VAMovie new.
	director := (VADirector new name: 'Francis Ford Coppola'; directedMovie: movie; directedMovie: movie2; directedMovie: movie3 ; directedMovie: movie4; directedMovie: movie5 ).
	
	movie title: 'The Godfather'; 
		    director: director; 
		    duration: '175 minutes'; 
		    country: 'USA'; 
		    genre: 'Suspense'. 
	
	movie2 title: 'The Godfather 2';
			director: director;
			duration: '200 minutes';
			country: 'USA';
			genre: 'Drama'.
			
	movie3 title: 'The Godfather 3';
			director: director;
			duration: '163 minutes';
			country: 'USA';
			genre: 'Drama'.
			
	movie4 title: 'Apocalypse Now';
			director: director;
			duration: '153 minutes';
			country: 'USA';
			genre: 'Drama'.
			
	movie5 title: 'The Outsiders';
			director: director;
			duration: '91 minutes';
			country: 'USA';
			genre: 'Drama'.
	
	director visualize.! !

Object subclass: #VAMovie
	instanceVariableNames: 'title director duration country genre'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'VisuAule-Objects'!

!VAMovie methodsFor: 'accesing' stamp: 'CamiloGomez 7/6/2012 18:05'!
country: aString

	country := aString! !

!VAMovie methodsFor: 'accesing' stamp: 'CamiloGomez 7/6/2012 18:04'!
director: aDirector

	director := aDirector! !

!VAMovie methodsFor: 'accesing' stamp: 'CamiloGomez 7/6/2012 16:25'!
duration: aTime

	duration := aTime! !

!VAMovie methodsFor: 'accesing' stamp: 'CamiloGomez 7/6/2012 18:05'!
genre: aString

	genre := aString! !

!VAMovie methodsFor: 'initialize-release' stamp: 'CamiloGomez 7/17/2012 17:53'!
initialize

	super initialize.
	
	title := ''.
	director := VADirector new.
	duration := ''.
	country := ''.
	genre := ''.! !

!VAMovie methodsFor: 'accesing' stamp: 'CamiloGomez 7/6/2012 18:05'!
title: aString

	title := aString! !

Object subclass: #VAPerson
	instanceVariableNames: 'name'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'VisuAule-Objects'!

VAPerson subclass: #VADirector
	instanceVariableNames: 'directedMovies'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'VisuAule-Objects'!

!VADirector methodsFor: 'actions' stamp: 'CamiloGomez 7/6/2012 18:03'!
directedMovie: aMovie

	directedMovies add: aMovie.! !

!VADirector methodsFor: 'initialize-release' stamp: 'CamiloGomez 7/17/2012 17:50'!
initialize

	super initialize.
	directedMovies := OrderedCollection new.! !

!VADirector methodsFor: 'as yet unclassified' stamp: 'CamiloGomez 7/18/2012 15:40'!
shouldBeAddedAsElementOnEnter
	^ false! !

!VAPerson methodsFor: 'initialize-release' stamp: 'CamiloGomez 7/17/2012 17:27'!
initialize
	super initialize.
	
	name := ''.! !

!VAPerson methodsFor: 'actions' stamp: 'CamiloGomez 6/27/2012 15:25'!
name: aString
	name := aString! !

VAPerson subclass: #VAReader
	instanceVariableNames: 'readBooks'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'VisuAule-Objects'!

!VAReader methodsFor: 'initialize-release' stamp: 'CamiloGomez 7/6/2012 18:03'!
initialize

	super initialize.
	readBooks := OrderedCollection new.! !

!VAReader methodsFor: 'actions' stamp: 'CamiloGomez 7/6/2012 18:02'!
readBook: aBook
	readBooks add: aBook! !

!String methodsFor: '*visuaule-objects' stamp: 'CamiloGomez 7/26/2012 16:39'!
roassalElementFor: object
	| element |
	
	element := ROElement on: object.
	element + ROLabel + ROBorder.
	element translateTo: 5@5.
	element @ ROLightlyHighlightable @ RODraggable.
	
	^ element! !

!String methodsFor: '*visuaule-objects' stamp: 'CamiloGomez 6/27/2012 15:39'!
shouldBeAddedAsElement
	^ false! !

!Collection methodsFor: '*visuaule-objects' stamp: 'CamiloGomez 7/27/2012 17:56'!
addReferencedObjectsOf: elementForSelf on: view

	| el varNames count |
	
	(self size < 5)
	ifTrue: [
		"count := 1.
		self class allInstVarNames do: [ :varName |
			(self at: count) notNil
			ifTrue: [ el := self roassalElementFor: (self at: count) withName: varName. 
					(elementForSelf model) addElement: el of: elementForSelf on: view.
					count := count + 1.]
			ifFalse: [ count := count  + 1.]]"
		
		self do: [ :element | 
					element notNil
					ifTrue: [el := element roassalElementFor: element.
				 	(elementForSelf model) addElement: el of: elementForSelf on: view.]]] 
	ifFalse: []! !

!Collection methodsFor: '*visuaule-objects' stamp: 'CamiloGomez 8/14/2012 15:51'!
elementsNotAdded

	| bothColumns newElement varSize nameSize |
	
	bothColumns := OrderedCollection new.
	newElement := ROElement new.
	varSize := ROElement new.
	nameSize := ROElement new.
	
	nameSize := ((ROElement on: 'size') + ROLabel).
	varSize := (self roassalOtherElementFor: self size).
	
	bothColumns add: nameSize.
	bothColumns add: varSize.
	ROHorizontalLineLayout on: bothColumns.
	
	newElement addAll: bothColumns.
	
	newElement + ROBorder.
	newElement shapesDo: [ :d | d color: Color blue].
	
	^ newElement.
	
! !

!Collection methodsFor: '*visuaule-objects' stamp: 'CamiloGomez 8/14/2012 16:23'!
interactionFor: elementForSelf on: view

	| elementForSelf2 |

	(self size < 5)
	ifTrue: [super interactionFor: elementForSelf on: view]
	ifFalse: [
	| el view2 |
	
	view2 := ROView new.
	view2 title: 'VisuAule'.
	view2 @ RODraggable @ RODraggableWithVelocity.
	view2 on: ROMouseLeftClick do: [ :event | ROZoomInMove new on: view2].
	view2 on: ROMouseRightClick do: [ :event | ROZoomOutMove new on: view2].
	"view add: ((ROButtonElement named: 'Zoom in' do: [ROZoomInMove new on: view]) + ROBorder).
	view add: ((ROButtonElement named: 'Zoom out' do: [ROZoomOutMove new on: view]) + ROBorder)."	
	
	elementForSelf2 := self roassalElementFor: self.
	super interactionFor: elementForSelf on: view.
	elementForSelf @ (ROMenuActivable new item: 'visualize elements' action: [
			view2 add: elementForSelf2.
			super interactionFor: elementForSelf2 on: view2. 
			self do: [ :element | 
					element notNil
					ifTrue: [el := element roassalElementFor: element.
					(elementForSelf model) addElement: el of: elementForSelf2 on: view2.]].
			ROTreeLayout on: ((view2 elements) "select: [ :each | each model ~= elementForSelf2 model]").
"			ROHorizontalLineLayout on: ((view2 elements) select: [ :each | each model ~= elementForSelf2 model])."
			view2 open]).]! !
